# This workflow is a practical GIS extension of the building classification method of Sturrock et al (2018). It allows output of model train/test res/non-res attributes, and model predictions and recoded (i.e. classified to res=1 or non-res=0) predictions, to csv format tables for visualisation with subsequent GDAL workflow

setwd("/mainfs/scratch/username/sturrock2window/")

# Load workspace with Sturrock model output and packages
#load("/mainfs/scratch/username/sturrock2window/rworkspaceNGA.RData")



#to display data clearly
#structure_points_testNGAdf <- as.data.frame(structure_points_testNGA)
#sl_model_binomSL.predictdf <- as.data.frame(sl_model_binom$SL.predict)



# Prediction quantities from the superlearner model are returned in the form of probabilities. A cut off threshold is needed to determine binomial classification as a one or zero
# The threshold is determined in training with cross-validation
# Recode probabilities (using threshold as value obtained from trained model (opt_cutoff_SL))
# If p exceeds threshold, 1 else 0: res_or_nonres
# Use dplyr's ifelse() function to recode the probabilities to the threshold:

library(dplyr)			  
res_or_nonrespred <- ifelse(sl_model_binom$SL.predict>=opt_cutoff_SL$opt_cutoff,1,0)

NGApred1 <- as.data.frame(res_or_nonrespred)
NGApred2 <- as.data.frame(prediction_structure_pointsNGA)

library(data.table)
setDT(NGApred1, keep.rownames = "Row")
setDT(NGApred2, keep.rownames = "Row")

NGApred1$Row <- as.numeric(as.character(NGApred1$Row))
NGApred2$Row <- as.numeric(as.character(NGApred2$Row))

NGApred3 <- merge(NGApred1, NGApred2, by= "Row", all= TRUE)

# output dataframe to csv
write.csv(NGApred3, file = "/mainfs/scratch/username/sturrock2window/NGApred3.csv", sep = ",", na = "", row.names = TRUE, col.names = TRUE)


## and output raw probabilities attached to polygon ids, in addition to the above recode derived from the training data
NGApred0<- as.data.frame(sl_model_binom$SL.predict)
setDT(NGApred0, keep.rownames = "Row")
setDT(NGApred2, keep.rownames = "Row")

NGApred0$Row <- as.numeric(as.character(NGApred0$Row))
NGApred2$Row <- as.numeric(as.character(NGApred2$Row))

NGApred4 <- merge(NGApred0, NGApred2, by= "Row", all= TRUE)

# output dataframe to csv
write.csv(NGApred4, file = "/mainfs/scratch/username/sturrock2window/NGApred4.csv", sep = ",", na = "", row.names = TRUE, col.names = TRUE)


## output train/test res/non-res label attributes attached to polygon ids, in a similar fashion																							   
NGAtraintest <- as.data.frame(structure_pointsNGA)

setDT(NGAtraintest, keep.rownames = "Row")
NGAtraintest$Row <- as.numeric(as.character(NGAtraintest$Row))

# output dataframe to csv
write.csv(NGAtraintest, file = "/mainfs/scratch/username/sturrock2window/NGAtraintest.csv", sep = ",", na = "", row.names = TRUE, col.names = TRUE)
